header "Security Type Systems"

theory Sec_Type_Expr imports Big_Step
begin

subsection "Security Levels and Expressions"

type_synonym level = nat

text{* The security/confidentiality level of each variable is globally fixed
for simplicity. For the sake of examples --- the general theory does not rely
on it! --- a variable of length @{text n} has security level @{text n}: *}

definition sec :: "name \<Rightarrow> level" where 
  "sec n = size n"

fun sec_aexp :: "aexp \<Rightarrow> level" where
"sec_aexp (N n) = 0" |
"sec_aexp (V x) = sec x" |
"sec_aexp (Plus a1 a2) = max (sec_aexp a1) (sec_aexp a2)"

fun sec_bexp :: "bexp \<Rightarrow> level" where
"sec_bexp (B bv) = 0" |
"sec_bexp (Not b) = sec_bexp b" |
"sec_bexp (And b1 b2) = max (sec_bexp b1) (sec_bexp b2)" |
"sec_bexp (Less a1 a2) = max (sec_aexp a1) (sec_aexp a2)"


abbreviation eq_le :: "state \<Rightarrow> state \<Rightarrow> level \<Rightarrow> bool"
  ("(_ = _ '(\<le> _'))" [51,51,0] 50) where
"s = s' (\<le> l) == (\<forall> x. sec x \<le> l \<longrightarrow> s x = s' x)"

abbreviation eq_less :: "state \<Rightarrow> state \<Rightarrow> level \<Rightarrow> bool"
  ("(_ = _ '(< _'))" [51,51,0] 50) where
"s = s' (< l) == (\<forall> x. sec x < l \<longrightarrow> s x = s' x)"

lemma aval_eq_if_eq_le:
  "\<lbrakk> s1 = s2 (\<le> l);  sec_aexp a \<le> l \<rbrakk> \<Longrightarrow> aval a s1 = aval a s2"
by (induct a) auto

lemma bval_eq_if_eq_le:
  "\<lbrakk> s1 = s2 (\<le> l);  sec_bexp b \<le> l \<rbrakk> \<Longrightarrow> bval b s1 = bval b s2"
by (induct b) (auto simp add: aval_eq_if_eq_le)

end
