header "A Typed Language"

theory Types_Exp imports Complex_Main begin

(* The beginning of this exercise is a copy of IMP/Types.thy: *)

subsection "Arithmetic Expressions"

datatype val = Iv int | Rv real

type_synonym name = string
type_synonym state = "name \<Rightarrow> val"

datatype aexp =  Ic int | Rc real | V name | Plus aexp aexp

inductive taval :: "aexp \<Rightarrow> state \<Rightarrow> val \<Rightarrow> bool" where
"taval (Ic i) s (Iv i)" |
"taval (Rc r) s (Rv r)" |
"taval (V x) s (s x)" |
"taval a1 s (Iv i1) \<Longrightarrow> taval a2 s (Iv i2)
 \<Longrightarrow> taval (Plus a1 a2) s (Iv(i1+i2))" |
"taval a1 s (Rv r1) \<Longrightarrow> taval a2 s (Rv r2)
 \<Longrightarrow> taval (Plus a1 a2) s (Rv(r1+r2))" 

inductive_cases [elim!]:
  "taval (Ic i) s v"  "taval (Rc i) s v"
  "taval (V x) s v"
  "taval (Plus a1 a2) s v"

(* Write a function taval' :: "aexp \<Rightarrow> state \<Rightarrow> val option" that is equivalent to taval

   The option type is a standard datatype declared as

     datatype 'a option = None | Some 'a 

   Your function taval' should return None if the expression cannot be evaluated (e.g. 
   because of a type error), and Some v if the value of the expression is v.
*)

fun taval' :: "aexp \<Rightarrow> state \<Rightarrow> val option" where
  "taval' (Ic i) s = Some (Iv i)"
(* ... *)

(* state and prove correctness of taval' *)



(* The second part is a copy of IMP/Types.thy again: *)
subsection "Boolean Expressions"

datatype bexp = B bool | Not bexp | And bexp bexp | Less aexp aexp

inductive tbval :: "bexp \<Rightarrow> state \<Rightarrow> bool \<Rightarrow> bool" where
"tbval (B bv) s bv" |
"tbval b s bv \<Longrightarrow> tbval (Not b) s (\<not> bv)" |
"tbval b1 s bv1 \<Longrightarrow> tbval b2 s bv2 \<Longrightarrow> tbval (And b1 b2) s (bv1 & bv2)" |
"taval a1 s (Iv i1) \<Longrightarrow> taval a2 s (Iv i2) \<Longrightarrow> tbval (Less a1 a2) s (i1 < i2)" |
"taval a1 s (Rv r1) \<Longrightarrow> taval a2 s (Rv r2) \<Longrightarrow> tbval (Less a1 a2) s (r1 < r2)"

inductive_cases tbval_cases:
  "tbval (B b) s v"
  "tbval (Not b) s v"
  "tbval (And b1 b2) s v"
  "tbval (Less a1 a2) s v"

(* Repeat the exercise for boolean expressions. 
   State and prove correctness of your function. *)

end